## Chapter 11 
library(stats)
R0 <- 1.6
k <- 0.1
calc_q <- function(q) {(q * ((1 + (R0/k)*(1-q))^k)) - 1}
results <- uniroot(calc_q, interval=c(0.1, 0.999999), maxiter=100000)
q <- results$root

##入国者別大規模流行確率p
narrivals <- c(1000, 2000, 4000, 8000)
prevalence <- c(0.001, 0.002, 0.005)
arrivemat <- matrix(rep(narrivals), length(narrivals), length(prevalence))
n <- sweep(arrivemat, 2, prevalence, FUN = '*')
rownames(n) <- narrivals
colnames(n) <- prevalence

p <- 1-(q^n)

## 流行収束の判定
library(dplyr)
library(magrittr)


# 感染者の発症日
onsets <- as.Date(c("2020-02-03", "2020-02-13", "2020-02-15", "2020-02-17", "2020-02-17", "2020-02-17", "2020-02-19", "2020-02-19", "2020-02-19", 
            "2020-02-20", "2020-02-20", "2020-02-20", "2020-02-20", "2020-02-20", "2020-02-21", "2020-02-22", "2020-02-22", "2020-02-23", 
            "2020-02-23", "2020-02-23", "2020-02-24", "2020-02-24", "2020-02-24", "2020-02-24", "2020-02-24", "2020-02-25", "2020-02-26", 
            "2020-02-26", "2020-02-26", "2020-02-26", "2020-02-27", "2020-02-29", "2020-03-01", "2020-03-01", "2020-03-05", "2020-03-07", 
            "2020-03-08", "2020-03-09", "2020-03-09"))
latest_onset <- as.Date(max(onsets))

# R0とｋの値
R0_k <- c(0.5, 0.25)

# ガンマ分布を従う発症間隔のパラメータ、Nishiura他（2020）
mean_si = 4.7
sd_si = 2.9
si_par <- c(mean_si^2/sd_si^2, mean_si/sd_si^2)

# ワイブル分布を従う報告の遅れのパラメータ
rd_par <- c(1.741, 8.573)

# 負の二項分布
py <- function(y, R0, k, log=FALSE) { dnbinom(y, mu=R0, size=k, log=log) }

# 流行t日目における感染者が報告される確率
pr_newcases <- Vectorize(function(t, onset, R0_k, si_par, rd_par) {
    
  # 報告の遅れと発症間隔の分布
  h <- function(t, rd_par) { pweibull(t, rd_par[1], rd_par[2]) - pweibull(t-1, rd_par[1], rd_par[2]) }
  f <- function(t, si_par) { pgamma(t, si_par[1], si_par[2]) - pgamma(t-1, si_par[1], si_par[2]) }
    
  prodsum <- 1
  for (i in seq_len(length(onset))) { 
    ti <- onset[i]
    df <- data.frame(day = seq(as.numeric(ti-ti), as.numeric(t-ti), 1)) %>% mutate(ft = f(day, si_par), ht = h(day, rd_par))
    conv = c()
    conv <- sapply(1:nrow(df), function(j) c(conv, sum(df$ft[1:j] * df$ht[j:1])))
    df %<>% do(cbind(.,conv=conv))
    F <- sum(df$conv)
    y <- 0:250
    ysum <- sum(py(y=y, R0=R0_k[1], k=R0_k[2]) * F^y)
    prodsum <- prodsum * ysum
  }
    
  return(1-prodsum)    
}, vectorize.args=c("t"))

(daystocalc = 42) # 最新発症日から計算する日数

## 計算
dat <- data.frame(t=seq(latest_onset,(latest_onset+daystocalc),by="1 day")) 
results <- dat %>% mutate(prob=pr_newcases(t, onset=onsets, R0_k=R0_k, si_par=si_par, rd_par=rd_par))

## 描画
library(ggplot2)
library(lubridate)

mmddjp = function(x) paste0(sprintf("%02d",month(ymd(x))),"月",(sprintf("%02d",day(ymd(x)))),"日")
ggplot(results) + 
  geom_line(aes(x=t, y=prob*100)) +
  geom_hline(yintercept=5) +
  geom_text(aes(x=t[1]+5, y=5, label = "しきい値=5%"), vjust=-1, size=4) +
  scale_y_continuous("\n追加感染者確定の確率, %", expand = c(0,0), limits = c(0, 100)) +
  scale_x_date("\n報告日", date_breaks="10 days", labels=mmddjp, expand=c(0,0),    
               limits=c(results$t[1],results$t[1]+daystocalc)) +
  theme_classic()
